==> disk-file-writer-1.0.0.tgz: package/src/index.ts <==
import { workerBuilder } from "./WorkerBuilder";
import fileWorker from './file-worker';
import db from './db';
import webmWriter2 from './webm-writer2';

class EventCenter {
    eventMGR: Map<string, Set<(payload?: any) => void>>;
    on(eventName: string, callback: (payload?: any) => void) {
        const callbacks = this.eventMGR.get(eventName);
        if (!callbacks) {
            this.eventMGR.set(eventName, new Set([callback]));
            return;
        }
        callbacks.add(callback);
    }
    off(eventName: string, callback: (payload?: any) => void) {
        if (!callback) {
            this.eventMGR.delete(eventName);
            return;
        }
        const callbacks = this.eventMGR.get(eventName);
        if (!callbacks) {
            return;
        }
        callbacks.delete(callback);
    }
    emit(eventName: string, payload?: any) {
        const callbacks = this.eventMGR.get(eventName);
        callbacks.forEach(callback => {
            if (callback && typeof callback === 'function') {
                callback(payload);
            }
        })
    }
}

interface IFileWriter {
    fileSizeQuota: number; // byte
    maxDaysToKeepFile: number;
}

export class FileWriter extends EventCenter {
    handle: Worker;
    props: IFileWriter;
    constructor(props: IFileWriter) {
        super();
        this.props = props;
        this.init();
    }

    init() {
        this.handle = workerBuilder(fileWorker, [db, webmWriter2]);
        this.setupListener();
    }

    setupListener() {
        this.handle.onmessage = (ev: any) => {
            switch (ev.data.cmd) {
                case 'initOK':
                case 'createLogFileReady':
                case 'OPFSFilesInfo':
                    this.emit(ev.data.cmd, ev.data.payload);
                    break;
                default:
                    break;
            }
        }
    }

    async startWriteVideoFile(stream: MediaStream) {
        const fileHandle = await window.showSaveFilePicker({ 
            suggestedName: 'video.webm',
            types: [{
                description: 'video file',
                accept: { 
                    'video/webm': ['.webm']
                }
            }]
        });
        const videoTrack = stream.getVideoTracks()[0];
        const trackSettings = videoTrack.getSettings();
        const vtp = new MediaStreamTrackProcessor({
            track: videoTrack
        });
        const frameReadable = vtp.readable;

        this.handle.postMessage({
            cmd: 'startWriteVideoFile',
            fileHandle,
            frameReadable,
            trackSettings,
        })
    }

    stopWriteVideoFile() {
        this.handle.postMessage({
            cmd: 'stopWriteVideoFile'
        })
    }

    createLogFile(filenamePrefix: string, filenameExtension: string) {
        this.handle.postMessage({
            cmd: 'createLogFile',
            fileSizeQuota: this.props.fileSizeQuota,
            maxDaysToKeepFile: this.props.maxDaysToKeepFile,
            filenamePrefix: filenamePrefix,
            filenameExtension: filenameExtension
        })
    }

    writeLog(text: string) {
        this.handle.postMessage({
            cmd: 'logs',
            text
        })
    }

    saveLogFile() {
        this.handle.postMessage({
            cmd: 'saveLogFile'
        });
    }

    getOPFSFilesInfoByDate(startDate: Date, endDate: Date) {
        this.handle.postMessage({
            cmd: 'getOPFSFilesInfoByDate',
            startDate,
            endDate
        });
    }

    async getOPFSFileInfoByName(filename: string) {
        const rootDirHandle = await navigator.storage.getDirectory();
        const fileHandle = await rootDirHandle.getFileHandle(filename);
        return await fileHandle.getFile();
    }

    async moveOPFSFileByDate(startDate: Date, endDate: Date) {
        const targetDirHandle = await window.showDirectoryPicker({
            mode: 'readwrite'
        });

        this.handle.postMessage({
            cmd: 'moveOPFSFileByDate',
            targetDirHandle,
            startDate,
            endDate
        })
    }

    async writeRealTimeLog (fileName: string = 'realtimeLog', fileExtension: string = '.log') {
        const targetFileHandle = await window.showSaveFilePicker({
            suggestedName: fileName,
            types: [{
                description: 'real time log',
                accept: {
                    'text/plain': [fileExtension]
                }
            }]
        });

        this.handle.postMessage({
            cmd: 'writeRealTimeLog',
            targetFileHandle
        });
    }

    stopWriteRealTimeLog() {
        this.handle.postMessage({
            cmd: 'stopWriteRealTimeLog'
        })
    }

    clearOPFSFiles() {
        this.handle.postMessage({
            cmd: 'clearOPFSFiles'
        })
    }
}

==> disk-file-writer-1.0.1.tgz: package/src/index.ts <==
import { workerBuilder } from "./WorkerBuilder";
import fileWorker from './file-worker';
import db from './db';
import webmWriter2 from './webm-writer2';

class EventCenter {
    eventMGR: Map<string, Set<(payload?: any) => void>>;
    on(eventName: string, callback: (payload?: any) => void) {
        const callbacks = this.eventMGR.get(eventName);
        if (!callbacks) {
            this.eventMGR.set(eventName, new Set([callback]));
            return;
        }
        callbacks.add(callback);
    }
    off(eventName: string, callback: (payload?: any) => void) {
        if (!callback) {
            this.eventMGR.delete(eventName);
            return;
        }
        const callbacks = this.eventMGR.get(eventName);
        if (!callbacks) {
            return;
        }
        callbacks.delete(callback);
    }
    emit(eventName: string, payload?: any) {
        const callbacks = this.eventMGR.get(eventName);
        callbacks.forEach(callback => {
            if (callback && typeof callback === 'function') {
                callback(payload);
            }
        })
    }
}

interface IFileWriter {
    fileSizeQuota: number; // byte
    maxDaysToKeepFile: number;
}

type MediaSrouce = MediaStream | HTMLAudioElement;

type startWriteVideoFileParams = {
    videoStream?: MediaStream;
    shareScreen?: boolean;
    recordMicrophone?: boolean;
}

export class FileWriter extends EventCenter {
    handle: Worker;
    props: IFileWriter;
    audioSrouces: Set<MediaSrouce>;
    audioContext: AudioContext;
    audioDestinationNode: MediaStreamAudioDestinationNode;
    defaultArgs: {
        videoFolderPath: '/User/Zoom/Local'
    }
    constructor(props: IFileWriter) {
        super();
        this.props = props;
        this.init();
    }

    init() {
        this.handle = workerBuilder(fileWorker, [db, webmWriter2]);
        this.audioContext = new AudioContext();
        this.audioDestinationNode = this.audioContext.createMediaStreamDestination();
        this.setupListener();
    }

    setupListener() {
        this.handle.onmessage = (ev: any) => {
            switch (ev.data.cmd) {
                case 'initOK':
                case 'createLogFileReady':
                case 'OPFSFilesInfo':
                    this.emit(ev.data.cmd, ev.data.payload);
                    break;
                default:
                    break;
            }
        }
    }

    addAudioSource(source: MediaSrouce) {
        let audioSrouceNode: MediaElementAudioSourceNode | MediaStreamAudioSourceNode;
        if ('controls' in source) {
            // HTMLAudioElement
            audioSrouceNode = this.audioContext.createMediaElementSource(source);
        } else if (typeof source.getTracks === 'function') {
            // MediaStream
            audioSrouceNode = this.audioContext.createMediaStreamSource(source);
        }
        audioSrouceNode.connect(this.audioDestinationNode);
    }

    async startWriteVideoFile({ videoStream, shareScreen, recordMicrophone  }: startWriteVideoFileParams) {
        let _videoStream = videoStream;
        let _audioStream = this.audioDestinationNode.stream;

        const fileHandle = await window.showSaveFilePicker({ 
            suggestedName: 'video.webm',
            types: [{
                description: 'video file',
                accept: { 
                    'video/webm': ['.webm']
                }
            }]
        });

        if (videoStream && typeof videoStream.getTracks !== 'function') {
            throw new Error('the stream you passed in is not a valid MediaStream');
        }

        if (shareScreen) {
            _videoStream = await navigator.mediaDevices.getDisplayMedia({
                video: {
                    width: 800,
                    frameRate: 30
                },
                audio: true
            })
        }

        if (recordMicrophone) {
            _audioStream = await navigator.mediaDevices.getUserMedia({
                audio: {
                    sampleRate: 44100,
                    echoCancellation: true,
                    noiseSuppression: true
                },
                video: false
            });
        }

        this.handle.postMessage({
            cmd: 'startWriteVideoFile',
            fileHandle,
            videoStream: _videoStream,
            audioStream: _audioStream,
        })
    }

    stopWriteVideoFile() {
        this.handle.postMessage({
            cmd: 'stopWriteVideoFile'
        })
    }

    createLogFile(filenamePrefix: string, filenameExtension: string) {
        this.handle.postMessage({
            cmd: 'createLogFile',
            fileSizeQuota: this.props.fileSizeQuota,
            maxDaysToKeepFile: this.props.maxDaysToKeepFile,
            filenamePrefix: filenamePrefix,
            filenameExtension: filenameExtension
        })
    }

    writeLog(text: string) {
        this.handle.postMessage({
            cmd: 'logs',
            text
        })
    }

    saveLogFile() {
        this.handle.postMessage({
            cmd: 'saveLogFile'
        });
    }

    getOPFSFilesInfoByDate(startDate: Date, endDate: Date) {
        this.handle.postMessage({
            cmd: 'getOPFSFilesInfoByDate',
            startDate,
            endDate
        });
    }

    async getOPFSFileInfoByName(filename: string) {
        const rootDirHandle = await navigator.storage.getDirectory();
        const fileHandle = await rootDirHandle.getFileHandle(filename);
        return await fileHandle.getFile();
    }

    async moveOPFSFileByDate(startDate: Date, endDate: Date) {
        const targetDirHandle = await window.showDirectoryPicker({
            mode: 'readwrite'
        });

        this.handle.postMessage({
            cmd: 'moveOPFSFileByDate',
            targetDirHandle,
            startDate,
            endDate
        })
    }

    async writeRealTimeLog (fileName: string = 'realtimeLog', fileExtension: string = '.log') {
        const targetFileHandle = await window.showSaveFilePicker({
            suggestedName: fileName,
            types: [{
                description: 'real time log',
                accept: {
                    'text/plain': [fileExtension]
                }
            }]
        });

        this.handle.postMessage({
            cmd: 'writeRealTimeLog',
            targetFileHandle
        });
    }

    stopWriteRealTimeLog() {
        this.handle.postMessage({
            cmd: 'stopWriteRealTimeLog'
        })
    }

    clearOPFSFiles() {
        this.handle.postMessage({
            cmd: 'clearOPFSFiles'
        })
    }
}
